/*
 * sound-plugin-hal-clovertrail
 *
 * Copyright (C) 2013  Intel Corporation. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <mm_debug.h>
#include "lpe_ctl.h"
#define INTEL_SST_CTRL "/dev/intel_sst_ctrl"

static int prepare_module_header(snd_ppp_params_t *ppp_params, int algo_id, int stream_id, int enable_status)
{
    ppp_params->str_id   = stream_id;
    ppp_params->algo_id  = algo_id;
    ppp_params->enable   = enable_status;
    ppp_params->size     = 0;
    ppp_params->params = (unsigned char *)ppp_params + sizeof(snd_ppp_params_t);
    return 0;
}

static int add_parameter_blocks(snd_ppp_params_t *ppp_params, int param_type, int param_length, char *param_data)
{
    unsigned char *uptr;
    ipc_ia_params_block_t *pbptr;

    uptr = (unsigned char *)ppp_params->params + ppp_params->size;
    pbptr = (ipc_ia_params_block_t *)uptr;
    pbptr->type = param_type;
    pbptr->size = param_length;
    memcpy(pbptr->params, param_data, param_length);
    ppp_params->size += sizeof(ipc_ia_params_block_t) + param_length;
    return 0;
}

static int set_parameters(snd_ppp_params_t *ppp_params)
{
    int ret, handle;

    handle = open(INTEL_SST_CTRL, O_RDWR);
    if ( handle < 0) {
        debug_log("%s open failed: %s", INTEL_SST_CTRL, strerror(errno));
        return -1;
    }

    ret = ioctl(handle, SNDRV_SST_SET_ALGO, ppp_params);
    if (ret != 0) {
        debug_log("ioctl cmd SET_ALGO failed: %s", strerror(errno));
        ret = -1;
    }
    debug_log("set parameters success");
    close(handle);
    return ret;

}

static int get_parameters(snd_ppp_params_t *ppp_params)
{
    int ret, handle;
    unsigned char * charp;
    unsigned int i, dump_size=0;
    handle = open(INTEL_SST_CTRL, O_RDWR);
    if (handle < 0) {
        debug_log("%s open failed: %s", INTEL_SST_CTRL, strerror(errno));
        return -1;
    }
    ret = ioctl(handle, SNDRV_SST_GET_ALGO, ppp_params);
    if ( ret != 0) {
        debug_log("ioctl cmd GET_ALGO failed");
        ret = -1;
    }
    close(handle);

    if (!ret) {
        debug_log("get parameters success: Algo ID 0x%x Str ID %d, Enable %d, Operation %d, Size %d\n",
             ppp_params->algo_id, ppp_params->str_id, ppp_params->enable, ppp_params->reserved, ppp_params->size);
        debug_log("Dump recieved data, size=%d \n", ppp_params->size);
        dump_size = ppp_params->size;
        charp = (unsigned char *)ppp_params->params;
        for (i = 0; i < dump_size; i++, charp++)
            debug_log("%02x ", *charp);
    }
    return ret;
}

int lpe_get_parameter(int str_id, int algo_id, int param_id, int param_length)
{
    unsigned int param_data[64];
    char acparams[64];
    snd_ppp_params_t *ppp_params = (snd_ppp_params_t *)acparams;

    debug_log("lpe_get_parameter: str_id=0x%x, algo_id=0x%x, param_id=0x%x, param_length=%d\n",
            str_id, algo_id, param_id, param_length);
    // Prepare header and add data to parameter block
    prepare_module_header(ppp_params, algo_id, str_id, DONT_CHANGE_MOD_STATE);
    add_parameter_blocks(ppp_params, param_id, param_length, (char*)param_data);

    if (get_parameters(ppp_params) != 0) {
        debug_log("Unable to get parameter\n");
        return -1;
    }
    return 0;
}

int lpe_set_parameter(int str_id, int algo_id, int param_id, int param_length, char *param_data)
{
    char acparams[32];
    snd_ppp_params_t *ppp_params = (snd_ppp_params_t *)acparams;

    debug_log("lpe_set_parameter: str_id=0x%x, algo_id=0x%x, param_id=0x%x, param_length=%d\n",
            str_id, algo_id, param_id, param_length);
    // Prepare header and add data to parameter block
    prepare_module_header(ppp_params, algo_id, str_id, DONT_CHANGE_MOD_STATE);
    add_parameter_blocks(ppp_params, param_id, param_length, param_data);

    if (set_parameters(ppp_params) != 0) {
        debug_log("Unable to set parameter\n");
        return -1;
    }
    return 0;
}

int change_algo_state(int str_id, int algo_id, int state)
{
    char acparams[64];
    snd_ppp_params_t *ppp_params = (snd_ppp_params_t *)acparams;

    debug_log("change_algo_state: str_id=0x%x, algo_id=0x%x, state=%d\n", str_id, algo_id, state);
    prepare_module_header(ppp_params, algo_id, str_id, state);
    if (set_parameters(ppp_params) != 0) {
        debug_log("Unable to set parameter\n");
        return -1;
    }
    return 0;
}

int check_algo_state(int str_id, int algo_id)
{
    char acparams[64];
    snd_ppp_params_t *ppp_params = (snd_ppp_params_t *)acparams;

    debug_log("check_algo_state: str_id=0x%x, algo_id=0x%x\n", str_id, algo_id);
    prepare_module_header(ppp_params, algo_id, str_id, DONT_CHANGE_MOD_STATE);
    if (get_parameters(ppp_params) != 0) {
        debug_log("Unable to get parameter\n");
        return -1;
    }
    if (ppp_params->enable) {
        debug_log ("State: enable\n");
    } else {
        debug_log ("State: disable\n");
    }
    return 0;
}

int lpe_setup_clv()
{
    //unsigned int param_data[32];

    /*debug_log("CLV: Enable MIC1_PATH/GAIN_CTRL\n");
    change_algo_state(0x05, LPE_ALGO_TYPE_VOL_CTRL, ENABLE);
    debug_log("CLV: Enable MIC2_PATH/GAIN_CTRL\n");
    change_algo_state(0x25, LPE_ALGO_TYPE_VOL_CTRL, ENABLE);

    param_data[0]=20;
    debug_log("Set MIC1_PATH/GAIN_CTRL/GAIN to value 20\n");
    lpe_set_parameter(0x05, LPE_ALGO_TYPE_VOL_CTRL, ALGO_PARAM_VOL_CTRL_GAIN, 1, (char *)param_data);
    debug_log("Set MIC2_PATH/GAIN_CTRL/GAIN to value 20\n");
    lpe_set_parameter(0x25, LPE_ALGO_TYPE_VOL_CTRL, ALGO_PARAM_VOL_CTRL_GAIN, 1, (char *)param_data);*/

    debug_log("CLV: Disable IHF_PATH/SDRC_CTRL\n");
    change_algo_state(0x21, 0x72, DISABLE);

    return 0;
}
